function S = create_trellis(memory)
% function S = create_trellis(memory)
%
% Creates a structure object in ans containing various state tables for the
% RSC encoder function rsc()
%
% memory should be the number of delay units in the RSC encoder (ie a 16
% state encoder would have a memory of 4)
%
% S.memory is the number of delay units in the RSC encoder
% S.one is a vector corresponding to the next state if a 1 is input
% S.one_out_ap is a vector corresponding to the output bit of the RSC
% encoder if a 1 is input and 0,1 become -1,+1
% S.zero is a vector corresponding to the next state if a 0 is input
% S.zero_out_ap is a vector corresponding to the output bit of the RSC
% encoder if a 0 is input and 0,1 become -1,+1
% S.one_inv is a vector corresponding to the previous state if a 1 was
% input
% S.zero_inv is a vector corresponding to the previous state if a 0 was
% input
% S.term gives the bits necessary to terminate the trellis
% 
%

S.memory = memory;

% generate the trellis state table
for ii=0:2^memory-1
    [y0 state0] = rsc(0,ii);
    [y1 state1] = rsc(1,ii);
    S.one(ii+1) = state1;
    S.zero(ii+1) = state0;
    y0(y0==0) = -1;
    y1(y1==0) = -1;
    S.one_out_ap(ii+1) = y1;
    S.zero_out_ap(ii+1) = y0;
end

% generate the inverse table
[temp S.one_inv] = sort(S.one);
S.one_inv=S.one_inv-1;
[temp S.zero_inv] = sort(S.zero);
S.zero_inv=S.zero_inv-1;

% generate the terminiation cell table
S.term = term_trellis(0,memory,S);

end


function ans = term_trellis(state,depth,S)
% function ans = term_trellis(state,depth)
%
% state should be 0 when you call this, this function is called recursively
% and state is one of the parameters
% depth should be amount of memory in decoder
% (ie, a 16 state decoder has a memory of 4)

% persistent holds the value between different function calls but the scope
% of the variable is only within this function (similar to static in C)
% ca is a cell array with an entry for every state with a vector of bits
% telling how to terminate the trellis to state 0
persistent ca;


% initialize if state==0
if state==0
    % clear ca, no other way to do this
    ca = [];
    % get the first state, the direction is always going to be from a 1
    % input since a 0 would keep a RSC encoder at state 0
    ca{S.one_inv(1)+1} = 1;
    % recursive call on first state to work with
    term_trellis(S.one_inv(1),depth-1,S);
    
elseif depth>0
    % get the children
    plus_child = S.one_inv(state+1);
    minus_child = S.zero_inv(state+1);
    % add the bits on how the children got there
    ca{plus_child+1} = [1 ca{state+1}];
    ca{minus_child+1} = [0 ca{state+1}];
    % recursive calls
    term_trellis(plus_child,depth-1,S);
    term_trellis(minus_child,depth-1,S);
end

% set return value
ans=ca;
end
