function ans = turbo_trial(EbN0dB,N,max_iter,S)
% completes a single turbo coder trial
%
% uses parallel contenated (PCCC) turbo encoding and decoding
% no puncturing is used, giving a code rate of r=1/3
% the interleaver is generated randomly each trial
% channel is AWGN, with Eb/N0 value specified
%
% EbN0dB gives the value of Eb/N0 in dB (reasonable values are 1 - 4 dB)
% generates a random binary information signal of length N
% signal length may grow by up to 4 bits to ensure trellis termination
% max_iter is the maximum number of iterations on the decoder
% ans is a vector of bit error rate values after each iteration (starting
% with uncoded case!)
% 

x = rand(1,N);
x = (x>.5);
x=double(x);
len_x = length(x);

% RSC encoder 1
% reset the state variable
[enc1 endstate] = rsc_vector(x);
%fprintf('Initial endstate: %d\n',endstate);
% terminate the trellis on encoder 1
if endstate~=0
    x = [x S.term{endstate+1}];
    len_x = length(x);
    [enc1 endstate] = rsc_vector(x);
%    fprintf('New endstate (should be terminated): %d\n', endstate);
end

% RSC encoder 2

% generate interleaving and deinterleaving table
% 1) generate vector interleave from 1 to len_x, representing indcicies
% 2) enter a loop from len_x to 1
%    a) generate a random number between 1 and len_x
%    b) swap the current position with the random number
interleave=1:len_x;
for ii=1:len_x
    num = round((len_x-1)*rand+1);
    temp = interleave(ii);
    interleave(ii) = interleave(num);
    interleave(num) = temp;
end
% use sort function to generate deinterleaving table
[sorted deinterleave] = sort(interleave);

xi = zeros(1,len_x);
% generate xi, the interleaved version of x
for ii=1:len_x
    xi(ii) = x(interleave(ii));
end

% encode the interleaved stream
enc2 = rsc_vector(xi);

% multiplex the 3 streams
y = zeros(1,3*len_x);
for ii=1:len_x
    y(3*(ii-1)+1) = x(ii);
    y(3*(ii-1)+2) = enc1(ii);
    y(3*(ii-1)+3) = enc2(ii);
end

% Binary PSK (antipodal) modulation
y(y==0) = -1;
% add in noise
% below was correct in IDEAL case
%EbN0 = 10^(EbN0dB/10);
%var = 1/(2*EbN0);

% base it on WE Ryan paper
% convert from decibels
EbN0 = 10^(EbN0dB/10);
% code rate is 1/3
EcN0 = EbN0/3;
% calculate Lc value
Lc = 4/EcN0;
% calculate variance
awgn_var = 1/(2*EcN0);
% add in AWGN with 0 mean and variance 
y = y + sqrt(awgn_var)*randn(1,3*len_x);


%fprintf('EbN0(dB)=%f EcN0(db)=%f EcN0=%f var=%f Lc=%f\n',EbN0dB,10*log10(EcN0),EcN0,awgn_var,Lc); 

% decoder
% steps:
% demultiplex the steams
% initialize the recursion
% begin the recursion
% make a decision

% demultiplex the 3 streams
ys = y(1:3:end);
e1n = y(2:3:end);
e2n = y(3:3:end);


% initialize the decoder
alpha1 = zeros(len_x,16);
alpha1(1,1) = 1;
beta1=zeros(len_x,16);
beta1(len_x,1) = 1;
Le21 = zeros(1,len_x);
alpha2 = zeros(len_x,16);
alpha2(1,1) = 1;
beta2 = [];

% do a hard decision based on input (represents uncoded case)
ydecoded = double(ys>0);
errors=sum(ydecoded~=x);
ans = zeros(1,11);
ans(1) = errors/len_x;
%fprintf('Iteration 0 BER=%f errors=%d\n',errors/len_x,errors);

for ii=1:max_iter
    % decoder 1
    [alpha1 beta1 Le12] = turbo_half_decode(ys, e1n, alpha1, beta1, Le21(deinterleave),Lc,S);
    % decoder 2
    [alpha2 beta2 Le21] = turbo_half_decode(ys(interleave), e2n, alpha2, beta2, Le12(interleave),Lc,S);
    % make a decision!
    L1 = Lc*ys+Le21(deinterleave)+Le12;
    ydecoded = double(L1>0);
    errors=sum(ydecoded~=x);
%    fprintf('Iteration %d BER=%f errors=%d\n',ii,errors/len_x,errors);
    ans(ii+1) = errors/len_x;
    % speed things up by breaking once the signal is error free
    if errors==0
        break;
    end
end